@@ -1,8 +1,8 @@
 from .sys import Console
 
 FILE_EXT = '.gamut'
-AUDIO_FORMATS = ['.wav', '.aif', '.aiff']
-MIME_TYPES = ['audio/x-wav', 'audio/x-aiff']
+AUDIO_FORMATS = ['.wav', '.aif', '.aiff', '.mp3']
+MIME_TYPES = ['audio/x-wav', 'audio/x-aiff', 'audio/mpeg']
 CONSOLE = Console()
 ANALYSIS_TYPES = ['timbre', 'pitch']
 ENVELOPE_TYPES = [
