@@ -28,9 +28,6 @@ from abc import ABC, abstractmethod
 
 # numpy
 import numpy as np
-from typing import NewType
-
-EnvelopeType = NewType("EnvelopeType", Envelope)
 
 
 class Analyzer(ABC):
@@ -94,7 +91,7 @@ class Analyzer(ABC):
         self.portable = portable
         self.name = splitext(basename(output))[0]
         output_dir = splitext(output)[0]
-        CONSOLE.log_disk_op(f'{"" if portable else "non-"}portable {self.type}', f'{basename(output_dir)}{FILE_EXT}').print()
+        CONSOLE.log_disk_op(f'{"" if portable else "non-"}portable {self.type}', f'{realpath(output_dir)}{FILE_EXT}').print()
         serialized_object = self._serialize()
 
         # write file and set correct file extension
@@ -180,7 +177,7 @@ class Corpus(Analyzer):
     source: str | list | None = None
         Source file(s) from which to build the ``Corpus``. ``source`` can be either a ``str`` or a list of ``str``, where 
         ``str`` is an audio file path or a directory of audio files. Note that, for any directory path, ``Corpus`` will recursively look for
-        any ``.wav`` or ``.aif`` audio files in folders and subfolders.
+        any ``.wav``, ``.aif``, or ``.mp3`` audio files in folders and subfolders.
 
     max_duration: str | list | None = None
         Maximum audio file duration to use in ``Corpus``. Applies to all audio files found in ``source``.
@@ -239,7 +236,7 @@ class Corpus(Analyzer):
         """ build corpus from `source` """
         st = time()
         CONSOLE.log_process('\N{brain} Building audio corpus...').print()
-        data = self.__compile()
+        data = self.__compile(source=None, data=[], excluded_files=[])
         CONSOLE.counter.finish()
         self.__set_source_root()
         self.tree.build(data=data, vector_path='features')
@@ -259,28 +256,28 @@ class Corpus(Analyzer):
             f'sources ({len(self.soundfiles)})': filenames,
         }
 
-    def __compile(self, source: list | str | None = None, excuded_files: list = [], data: list = []) -> None:
+    def __compile(self, source: list | str | None = None, excluded_files: list = [], data: list = []) -> None:
         """ recursively collects and extracts features from all audio files in `source` """
         source = source or self.source
         source_type = type(source or self.source)
         if source_type == list:
             for x in source:
-                self.__compile(x, excuded_files)
+                self.__compile(x, excluded_files=excluded_files, data=data)
             return data
 
         source_path = realpath(source)
         if isdir(source_path):
             for root, _, files in walk(source_path):
                 for f in files:
-                    self.__compile(join(root, f), excuded_files)
+                    self.__compile(join(root, f), excluded_files=excluded_files, data=data)
             return data
 
         filename = splitext(basename(source))[0]
         kind = filetype.guess(source_path)
-        if kind is None or kind.mime not in MIME_TYPES or filename in excuded_files:
+        if kind is None or kind.mime not in MIME_TYPES or filename in excluded_files:
             return data
 
-        excuded_files.append(filename)
+        excluded_files.append(filename)
 
         y, sr = load(path=source, sr=None, mono=True,
                      duration=self.max_duration)
@@ -304,7 +301,7 @@ class Corpus(Analyzer):
     def print_feature_choices(cls) -> None:
         print(f'feature choices: {ANALYSIS_TYPES}')
 
-    def __set_source_root(self):
+    def __set_source_root(self) -> None:
         """ remove commonprefix from file paths to avoid size overhead when writing corpus to disk """
 
         if not self.soundfiles:
@@ -376,7 +373,7 @@ class Mosaic(Analyzer):
 
     def __init__(self,
                  target: str | None = None,
-                 corpus: Iterable | Corpus = None,
+                 corpus: Iterable | Corpus | None = None,
                  sr: int | None = None,
                  beat_unit: float | int | None = None,
                  *args,
@@ -398,7 +395,7 @@ class Mosaic(Analyzer):
             self.features = corpora[0].features
             self.__build(corpora=corpora, sr=sr)
 
-    def __validate(self, target, corpus):
+    def __validate(self, target: str | None, corpus: Iterable | Corpus | None) -> None:
         if any([target, corpus]) and not all([target, corpus]):
             CONSOLE.error(
                 ValueError,
@@ -412,7 +409,7 @@ class Mosaic(Analyzer):
         if kind is None or kind.mime not in MIME_TYPES:
             CONSOLE.error(ValueError, f'{basename(target)} seems to be an invalid or unsupported audio file format.')
 
-    def __parse_corpus(self, corpus: Iterable | Corpus, corpora: Iterable):
+    def __parse_corpus(self, corpus: Iterable | Corpus, corpora: Iterable) -> Iterable:
         if not corpus:
             return
 
@@ -424,7 +421,7 @@ class Mosaic(Analyzer):
                 if prev_features != current_features:
                     CONSOLE.error(
                         ValueError,
-                        f'Corpus at index {i} has a different set of features ({c.features}) than corpus at index {i-1} ({corpus[i-1].features}). When using more than one corpus, make sure they are based on the same feature set.')
+                        f'Corpus at index {i} has a different set of features ({", ".join(c.features)}) than corpus at index {i-1} ({", ".join(corpus[i-1].features)}). When using more than one corpus, make sure they are based on the same feature set.')
                 prev_features = current_features
             # recursively unpack corpora
             for c in corpus:
@@ -438,7 +435,7 @@ class Mosaic(Analyzer):
     def __build(self, corpora: Iterable, sr: int | None = None) -> None:
         num_corpora = len(corpora)
         CONSOLE.log_process(
-            f'\N{brain} Building target from {"corpus" if num_corpora == 1 else f"{num_corpora} corpora"}...').print()
+            f'\N{brain} Building mosaic for {basename(self.target)} from {"corpus" if num_corpora == 1 else f"{num_corpora} corpora"}...').print()
         CONSOLE.log_subprocess('Loading target...').print()
         st = time()
         y, self.sr = load(self.target, sr=sr)
@@ -489,7 +486,7 @@ class Mosaic(Analyzer):
         CONSOLE.bar.finish()
         CONSOLE.elapsed_time(st).print()
 
-    def _serialize(self):
+    def _serialize(self) -> dict:
         mosaic = deepcopy(vars(self))
 
         # if not portable, delete audio samples from soundfiles on write
@@ -509,7 +506,7 @@ class Mosaic(Analyzer):
             "num. of grains": len(self.frames)
         }
 
-    def _preload(self, obj):
+    def _preload(self, obj: dict) -> dict:
         # reload soundfiles if non-portable
         if not obj['portable']:
             self.__load_soundfiles(obj['soundfiles'])
@@ -518,7 +515,7 @@ class Mosaic(Analyzer):
     def read(self, file: str) -> Self:
         return super().read(file, warn_user=self.frames)
 
-    def __load_soundfiles(self, soundfiles):
+    def __load_soundfiles(self, soundfiles: Iterable) -> None:
         CONSOLE.counter.message = CONSOLE.log_subprocess('Loading audio files: ')
         for corpus_id in soundfiles:
             corpus = soundfiles[corpus_id]
@@ -539,7 +536,7 @@ class Mosaic(Analyzer):
                  stretch_factor: float | int | Envelope | Iterable = 1.0,
                  onset_var: float | int | Envelope | Iterable = 0,
                  pan_depth: float | int | Envelope | Iterable = 5,
-                 grain_env: Envelope | str | Iterable = "cosine",
+                 grain_env: str | Envelope | Iterable = "cosine",
                  corpus_weights: float | int | Envelope | Iterable = 1.0,
 
                  # static parameters
